unit wsq_demo_main_form;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs,
  StdCtrls, ExtCtrls,
  LCLType, //LCLType for HBitmap
  IntfGraphics, // TLazIntfImage type
  fpImage; // TFPColor type

type


  { TMainForm }

  TMainForm = class(TForm)
    OpenButton: TButton;
    SaveButton: TButton;
    WSQtoStreamButton: TButton;
    RegisterButton: TButton;
    StreamToWSQButton: TButton;
    OutputImage: TImage;
    procedure ConverterButtonClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure OpenButtonClick(Sender: TObject);
    procedure RegisterButtonClick(Sender: TObject);
    procedure SaveButtonClick(Sender: TObject);
    procedure StreamToWSQButtonClick(Sender: TObject);
    procedure WSQtoStreamButtonClick(Sender: TObject);
  private
    { private declarations }
  public
    { public declarations }
    bitmap : HBitmap;
    open_dialog : TOpenDialog;
    save_dialog : TSaveDialog;
  end; 

{ WSQ library declarations }
function ReadImageFromFile (const file_name: PCHAR; width: PInteger; height: PInteger; imageData: PPCHAR): Integer; cdecl; external 'libWSQ_library64.so' name 'ReadImageFromFile';
function SaveImageToFile(const filename: PCHAR; filetype: Integer; width: Integer; height: Integer; imageData: PCHAR):Integer; cdecl; external 'libWSQ_library64.so' name 'SaveImageToFile';
function WSQ_decode_stream(input_data_stream: PCHAR; const input_stream_length: Integer; output_data_stream: PPCHAR; width: PInteger; height: PInteger; ppi: PInteger; comment_text: PPCHAR):Integer; cdecl; external 'libWSQ_library64.so' name 'WSQ_decode_stream';
function WSQ_encode_stream(input_data_stream: PCHAR; const width: Integer; const height: Integer; const bitrate:Double;
				   const ppi: Integer; comment_text: PCHAR; output_data_stream: PPCHAR; output_stream_length: PInteger):Integer; cdecl; external 'libWSQ_library64.so' name 'WSQ_encode_stream';



var
  MainForm: TMainForm;

   iFileHandle: Integer;
   input_data_stream, output_data_stream: PCHAR;
   input_data_stream2, output_data_stream2: PCHAR;
   image_width, image_height, ppi: Integer;
   bitrate: Double;
   comment_text: PCHAR;
   ret, n, input_stream_length: Integer;
   output_stream2_length: Integer;
   
   imagedata: PCHAR;
   width, height, number_of_pixels, pixel_number: Integer;
   TempIntfImg: TLazIntfImage;
   ImgHandle,ImgMaskHandle: HBitmap;
   x, y, row: Integer;
   CurColor: TFPColor;



implementation

{ TMainForm }

procedure TMainForm.FormCreate(Sender: TObject);
begin
       open_dialog := TOpenDialog.Create(MainForm);
       save_dialog := TSaveDialog.Create(MainForm);
       bitmap := HBitmap(NIL);
       comment_text := PCHAR(NIL); // It is important to initialize to NIL to avoid crashing

end;

procedure TMainForm.ConverterButtonClick(Sender: TObject);
begin

end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
        open_dialog.Destroy;
        save_dialog.Destroy;
end;

procedure TMainForm.OpenButtonClick(Sender: TObject);
begin


    open_dialog.Filter := 'All images|*.wsq;*.tif;*.png;*.jpg;*.bmp;*.tga;*.cel;*.rgb;*.gif;*.psd;*.iff;*.pcx;|' +
    'WSQ - FBI''s Wavelet Scalar Quantization (*.wsq)|*.wsq|' +
    'TIF - Tagged Information File Format (*.tif)|*.tif|' +
    'PNG - Portable network graphic images (*.png)|*.png|' +
    'Joint Photographic Experts Group images (*.jpg)|*.jpg|' +
    'Windows bitmaps (*.bmp)|*.bmp|' +
    'Truevision Targa Graphics images (*.tga)|*.tga|' +
    'Autodesk images (*.cel)|*.cel|' +
    'Silicon Graphics International true color images (*.rgb)|*.rgb|' +
    'Compuserve Graphic Interchange Format(*.gif)|*.gif|' +
    'Photoshop images (*.psd)|*.psd|' +
    'Amiga Bitmap Graphic (*.iff)|*.iff|' +
    'PCX - ZSoft Paintbrush images (*.pcx)|*.pcx|';


if open_dialog.Execute() then
begin


   ReadImageFromFile(PCHAR(open_dialog.FileName), @width, @height, @imagedata);

   number_of_pixels := width*height;

   TempIntfImg := TLazIntfImage.Create(width, height);
   OutputImage.Picture.Bitmap.Width := width;
   OutputImage.Picture.Bitmap.Height := height;
   TempIntfImg.LoadFromBitmap(OutputImage.Picture.Bitmap.Handle,OutputImage.Picture.Bitmap.MaskHandle);


     for y := 0 to TempIntfImg.Height-1 do begin
       row := width * y;
       for x := 0 to TempIntfImg.Width-1 do begin
         pixel_number := x + row;

         CurColor.Red := Word(Integer(imagedata[pixel_number])*$FF);
         CurColor.Green := Word(Integer(imagedata[pixel_number+number_of_pixels])*$FF);
         CurColor.Blue := Word(Integer(imagedata[pixel_number+2*number_of_pixels])*$FF);
         TempIntfImg.Colors[x,y] := CurColor;
       end;
      end;



//     OutputImage.Picture.Bitmap.Assign(TempIntfImg);
     OutputImage.Picture.Bitmap.LoadFromIntfImage(TempIntfImg);
     TempIntfImg.Free;


        if ( OutputImage.Picture.Bitmap.Handle <> HBitmap(NIL)) then
        begin
           SaveButton.Enabled := True;
           OutputImage.Height := OutputImage.Picture.Bitmap.Height;
           OutputImage.Width := OutputImage.Picture.Bitmap.Width;
           Invalidate;
        end
        else
        begin
            Application.MessageBox(PCHAR('Unable to load file'), PCHAR('WSQ library error'), MB_OK or MB_ICONERROR );
        end;




end;


end;

procedure TMainForm.RegisterButtonClick(Sender: TObject);
begin

//RegisterWSQ();
	ShowMessage('For registration please use program "registerWSQ"' + #13#10 + 'available in the same directory where WSQ Image Library' + #13#10 + '(libWSQ_library64.so) is located.' + #13#10 + 'Please visit http://www.cognaxon.com for more information.');


end;

procedure TMainForm.SaveButtonClick(Sender: TObject);
begin


    save_dialog.Filter := 'WSQ Files (*.wsq)|*.wsq|Windows BMP (*.bmp)|*.bmp|TIFF Files (*.tif)|*.tif|PNG Files (*.png)|*.png|' +
    'JPEG Files (*.jpg)|*.jpg|SGI Files (*.rgb)|*.rgb|Targa Files (*.tga)|*.tga';
    save_dialog.DefaultExt := 'wsq';


if save_dialog.Execute() then
begin


   number_of_pixels := OutputImage.Picture.Bitmap.Width * OutputImage.Picture.Bitmap.Height;
   imagedata := PChar(AllocMem(3*number_of_pixels + 1));


   TempIntfImg := TLazIntfImage.Create(0,0);
   TempIntfImg.LoadFromBitmap(OutputImage.Picture.Bitmap.Handle,OutputImage.Picture.Bitmap.MaskHandle);

     for y := 0 to TempIntfImg.Height-1 do begin
       row := TempIntfImg.Width * y;
       for x := 0 to TempIntfImg.Width-1 do begin
         pixel_number := x + row;

         CurColor := TempIntfImg.Colors[x,y];
         imagedata[pixel_number] := CHAR(CurColor.Red);
         imagedata[pixel_number+number_of_pixels] := CHAR(CurColor.Green);
         imagedata[pixel_number+2*number_of_pixels] := CHAR(CurColor.Blue);

       end;
      end;

      TempIntfImg.Free;


       //SetShowFilePropertiesDialog(1);
         SaveImageToFile(PCHAR(save_dialog.FileName), save_dialog.FilterIndex, OutputImage.Picture.Bitmap.Width, OutputImage.Picture.Bitmap.Height, imagedata);

end;


end;

procedure TMainForm.StreamToWSQButtonClick(Sender: TObject);
var
    x, y, row, row2, pixel_number, pixel_number2: Integer;
begin

   save_dialog.Filter := 'WSQ Files (*.wsq)|*.wsq';
    save_dialog.DefaultExt := 'wsq';

if save_dialog.Execute() then
begin
    bitrate := 0.75;
    ppi := 500;
//    comment_text := nil;
    comment_text := 'Cognaxon';

    if(input_data_stream2 <> PCHAR(nil)) then
    begin
    FreeMem(input_data_stream2); input_data_stream2 := PCHAR(nil);
    end;

    input_data_stream2 := PChar(AllocMem(image_width*image_height));


	// MAKE A MIRRORED/FLIPPED IMAGE (UP<->DOWN and LEFT<->RIGHT)
    for y := 0 to (image_height - 1) do
    begin
      row := image_width * y;
      row2 := image_width * (image_height - y - 1);

        for x := 0 to (image_width - 1) do
        begin
         pixel_number := row + x;
         pixel_number2 := row2 + (image_width - x - 1);

         input_data_stream2[pixel_number2] := output_data_stream[pixel_number];
        end;
    end;




// THE ENCODING OF DECOMPRESSED STREAM INTO WSQ STREAM
WSQ_encode_stream(input_data_stream2, image_width, image_height, bitrate, ppi, PCHAR(comment_text), @output_data_stream2, @output_stream2_length);

comment_text := PCHAR(NIL); // this prevents crashing when opening next WSQ image

    iFileHandle := FileCreate(save_dialog.FileName);
    FileWrite(iFileHandle, output_data_stream2^, output_stream2_length);
    FileClose(iFileHandle);


Application.MessageBox(PCHAR('Memory stream was compressed into another memory stream and saved to WSQ file.' + #13#10 + 'Now please press button "Open", select the saved file, open it and have a look at the' + #13#10 + 'results of memory stream processing. The processed image is mirrored/flipped' + #13#10 + '(UP<->DOWN and LEFT<->RIGHT).'), PCHAR('WSQ image library'), MB_OK );

end;


end;

procedure TMainForm.WSQtoStreamButtonClick(Sender: TObject);
var
  iFileLength: Integer;
begin


    open_dialog.Filter := 'FBI Wavelet Scalar Quantization (*.wsq)|*.wsq';


if open_dialog.Execute() then
begin

      iFileHandle := FileOpen(open_dialog.FileName, fmOpenRead);
      iFileLength := FileSeek(iFileHandle,0,2);
      FileSeek(iFileHandle,0,0);
      input_data_stream := PChar(AllocMem(iFileLength + 1));
      input_stream_length := FileRead(iFileHandle, input_data_stream^, iFileLength);
      FileClose(iFileHandle);


// THE DECODING OF WSQ STREAM INTO DECOMPRESSED STREAM
WSQ_decode_stream(input_data_stream, input_stream_length, @output_data_stream, @image_width, @image_height, @ppi, @comment_text);

//===========================
TempIntfImg := TLazIntfImage.Create(image_width, image_height);
OutputImage.Picture.Bitmap.Width := image_width;
OutputImage.Picture.Bitmap.Height := image_height;
TempIntfImg.LoadFromBitmap(OutputImage.Picture.Bitmap.Handle,OutputImage.Picture.Bitmap.MaskHandle);


  for y := 0 to TempIntfImg.Height-1 do begin
    row := image_width * y;

    for x := 0 to TempIntfImg.Width-1 do begin
      pixel_number := x + row;

      CurColor.Red := Word(Integer(output_data_stream[pixel_number])*$FF);
      CurColor.Green := Word(Integer(output_data_stream[pixel_number])*$FF);
      CurColor.Blue := Word(Integer(output_data_stream[pixel_number])*$FF);
      TempIntfImg.Colors[x,y] := CurColor;
    end;
   end;

//     OutputImage.Picture.Bitmap.Assign(TempIntfImg);
  OutputImage.Picture.Bitmap.LoadFromIntfImage(TempIntfImg);
  TempIntfImg.Free;

    if ( OutputImage.Picture.Bitmap.Handle <> HBitmap(NIL)) then
    begin
       SaveButton.Enabled := True;
       OutputImage.Height := OutputImage.Picture.Bitmap.Height;
       OutputImage.Width := OutputImage.Picture.Bitmap.Width;
       Invalidate;
    end
    else
    begin
        Application.MessageBox(PCHAR('Unable to load file'), PCHAR('WSQ library error'), MB_OK or MB_ICONERROR );
    end;
//===========================

       Application.MessageBox(PCHAR('WSQ file was loaded into memory stream and decompressed into another memory stream.' + #13#10 + 'Now please press button "stream -> WSQ" to compress memory stream and save it to file.' + #13#10 + 'WSQ compressed byte array was converted to HBITMAP for showing on screen.'), PCHAR('WSQ image library'), MB_OK );

       StreamToWSQButton.Enabled := True;
end;


end;

initialization
  {$I wsq_demo_main_form.lrs}

end.

